@@ -1,23 +1,38 @@
 
-## CB 2009/5,6
+## CB 2009/5,6,10
+
+
+.as_stm.simple_triplet_matrix <- 
+function(x, compat = FALSE) {
+    if (!inherits(x, "simple_triplet_matrix"))
+	stop("'x' not of class simple_triplet_matrix")
+    ## old-style
+    if(compat)
+	.structure(x$v, i = x$i, j = x$j, Dim = c(x$nrow, x$ncol),
+                   Dimnames = x$dimnames, class = "stm")
+    else
+	.Call("_as_stm", x$v, x$i, x$j, c(x$nrow, x$ncol), x$dimnames)
+}
 
 ## simple triple matrix C interfaces
 .sums_simple_triplet_matrix <- 
-function(x, DIM, na.rm) {
-    x <- structure(x$v, i = x$i, j = x$j, Dim = c(x$nrow, x$ncol),
-	Dimnames = x$dimnames, class = "stm")
+function(x, DIM, na.rm)
+{
+    x <- .as_stm.simple_triplet_matrix(x)
     .Call("_sums_stm", x, DIM, na.rm)
-
 }   
 
 .means_simple_triplet_matrix <-
-function(x, DIM, na.rm) {
-    x <- structure(x$v, i = x$i, j = x$j, Dim = c(x$nrow, x$ncol),
-	Dimnames = x$dimnames, class = "stm")
+function(x, DIM, na.rm)
+{
+    x <- .as_stm.simple_triplet_matrix(x)
     s <- .Call("_sums_stm", x, DIM, na.rm)
     if (na.rm) {
 	## FIXME inefficient
-	x[] <- is.na(x)
+	if (is.list(x))
+	    attr(x, "v")[] <- is.na(attr(x, "v"))
+	else
+	    x[] <- is.na(x)
 	s /(attr(x, "Dim")[-DIM] - .Call("_sums_stm", x, DIM, na.rm))
     } else
 	s / attr(x, "Dim")[-DIM]
@@ -28,7 +43,7 @@ function(x, DIM, na.rm) {
 
 rowSums <-
 function(x, na.rm = FALSE, dims = 1, ...)
-   UseMethod("rowSums") 
+    UseMethod("rowSums") 
 
 rowSums.default <-
 function(x, na.rm = FALSE, dims = 1, ...)
@@ -40,7 +55,7 @@ function(x, na.rm = FALSE, dims = 1, ...)
 
 colSums <-
 function(x, na.rm = FALSE, dims = 1, ...)
-   UseMethod("colSums") 
+    UseMethod("colSums") 
 
 colSums.default <-
 function(x, na.rm = FALSE, dims = 1, ...)
@@ -79,20 +94,26 @@ function(x, na.rm = FALSE, dims = 1, ...)
 ##      therefore has control over how to proceed. For now
 ##      it calls the bailout function below.
 tcrossprod.simple_triplet_matrix <-
-function(x, y = NULL) {
-    if(!inherits(x, "simple_triplet_matrix"))
-	stop("'x' not of class simple_triplet_matrix")
-    x <- structure(x$v, i = x$i, j = x$j, Dim = c(x$nrow, x$ncol),
-	Dimnames = x$dimnames, class = "stm")
+function(x, y = NULL)
+{
+    x <- .as_stm.simple_triplet_matrix(x)
     .Call("tcrossprod_stm_matrix", x, y,
-	 environment(tcrossprod.simple_triplet_matrix))
+          environment(tcrossprod.simple_triplet_matrix))
 }
 
 .tcrossprod.bailout <-
-function(x, y) {
+function(x, y)
+{
     t <- array(0, dim = attr(x, "Dim"), dimnames = attr(x, "Dimnames"))
-    t[cbind(attr(x, "i"), attr(x, "j"))] <- x
+    t[cbind(attr(x, "i"), attr(x, "j"))] <- 
+	if(is.list(x)) attr(x, "v") else x
     tcrossprod(t, y)
 }
 
+## KH 2009/10
+
+.structure <-
+function(x, ...)
+    `attributes<-`(x, c(attributes(x), list(...)))
+
 ###
